@@ -12,11 +12,17 @@ def open_():
 
     auth_data = load_auth_data()
     if auth_data is None:
-        click.secho('No auth data found, use "kks auth" to login and save contest id', fg='red', err=True)
+        click.secho(
+            'No auth data found, use "kks auth" to login and save contest id',
+            fg='red', err=True
+        )
         return
 
     if auth_data.login is None or auth_data.password is None:
-        click.secho('No password or login stored, opening contest without logging in', fg='yellow', err=True)
+        click.secho(
+            'No password or login stored, opening contest without logging in',
+            fg='yellow', err=True
+        )
 
     url = get_contest_url_with_creds(auth_data)
     click.secho('Opening... ', nl=False)
