@@ -4,6 +4,7 @@ from dataclasses import dataclass, field, fields
 from datetime import datetime, timedelta, timezone
 from enum import Enum
 from itertools import groupby
+from os import environ
 from typing import Optional
 from urllib.parse import parse_qs, urlsplit, quote as urlquote
 
@@ -21,9 +22,10 @@ CONTEST_ID_BY_GROUP = {}
 CONTEST_ID_BY_GROUP.update({
     f'19{group}': 130 + group for group in range(1, 12)
 })
-CONTEST_ID_BY_GROUP.update({
-    f'20{group}': int(f'20{group}') for group in range(1, 11)
-})
+for year in [20, 21]:
+    CONTEST_ID_BY_GROUP.update({
+        f'{year}{group}': int(f'{year}{group}') for group in range(1, 11)
+    })
 CONTEST_ID_BY_GROUP['free'] = 2021
 
 GROUP_ID_BY_CONTEST = {
@@ -37,8 +39,9 @@ MSK_TZ = timezone(timedelta(hours=3))
 
 
 class Links:
-    HOST = 'caos.myltsev.ru'
-    CGI_BIN = f'https://{HOST}/cgi-bin'
+    SCHEME = environ.get('KKS_CUSTOM_SCHEME', 'https')
+    DOMAIN = environ.get('KKS_CUSTOM_DOMAIN', 'caos.myltsev.ru')
+    CGI_BIN = f'{SCHEME}://{DOMAIN}/cgi-bin'
     WEB_CLIENT_ROOT = f'{CGI_BIN}/new-client'
 
 
@@ -63,9 +66,48 @@ class Status:
     REJECTED = 'Rejected'
     IGNORED = 'Ignored'
     PARTIAL = 'Partial solution'
+    DISQUALIFIED = 'Disqualified'
+    CHECK_FAILED = 'Check failed'
     NOT_SUBMITTED = 'Not submitted'
 
 
+class Lang(Enum):
+    def __new__(cls, value, suf, realname=None):
+        obj = object.__new__(cls)
+        obj._value_ = value
+        obj.realname = realname
+        obj.suf = suf
+        return obj
+
+    @property
+    def name(self):
+        if self.realname is not None:
+            return self.realname
+        return self._name_
+
+    # NOTE compiler ids may change
+    gcc = (2, '.c')
+    gxx = (3, '.cpp', 'g++')
+    python = (13, '.py')
+    perl = (14, '.pl')
+    ruby = (21, '.rb')
+    python3 = (23, '.py')
+    make = (25, '.tar')
+    gcc_vg = (28, '.c', 'gcc-vg')
+    gxx_vg = (29, '.cpp', 'g++-vg')
+    clang = (51, '.c')
+    clangxx = (52, '.cpp', 'clang++')
+    make_vg = (54, '.tar', 'make-vg')
+    gcc_32 = (57, '.c', 'gcc-32')
+    clang_32 = (61, '.c', 'clang-32')
+    clangxx_32 = (62, '.cpp', 'clang++-32')
+    gas_32 = (66, '.S', 'gas-32')
+    gas = (67, '.S')
+    rust = (70, '.rs')
+    gas_aarch64 = (101, '.S', 'gas-aarch64')
+    gas_armv7l = (102, '.S', 'gas-armv7l')
+
+
 class AuthData:
     def __init__(self, login, contest_id, password=None):
         self.login = login
@@ -243,15 +285,11 @@ class Submission:
         return self.status
 
     def suffix(self):
-        if self.compiler.startswith('gas'):
-            return '.S'
-        if '++' in self.compiler:
-            return '.cpp'
-        if self.compiler.startswith('make'):
-            return '.tar'
-        if self.compiler.startswith('gcc') or self.compiler.startswith('clang'):
-            return '.c'
-        return ''
+        compiler = self.compiler.replace('-', '_').replace('+', 'x')
+        try:
+            return getattr(Lang, compiler).suf
+        except AttributeError:
+            return ''
 
 
 class Report:
@@ -292,23 +330,33 @@ class Report:
 
 
 class TaskScore:
+    _colors = {
+        Status.REVIEW: 'green',
+        Status.OK: 'green',
+        # in standings CHECK has the same class/style as TESTING
+        Status.TESTING: 'bright_yellow',
+        Status.REJECTED: 'yellow',
+        Status.PARTIAL: 'red',
+        Status.CHECK_FAILED: 'bright_red',
+        Status.DISQUALIFIED: 'white',
+    }
+
     def __init__(self, contest: str, score: Optional[str], status: str):
         self.contest = contest
         self.score = score
         self.status = status
 
+    def bg_color(self):
+        if self.status == Status.DISQUALIFIED:
+            return 'bright_red'
+        return None
+
     def color(self):
-        return 'green' if self.status == Status.REVIEW \
-            else 'green' if self.status == Status.OK \
-            else 'bright_yellow' if self.status == Status.TESTING \
-            else 'yellow' if self.status == Status.REJECTED \
-            else 'red' if self.status == Status.PARTIAL \
-            else 'white'
-        # in standings CHECK has the same style as TESTING
+        return self._colors.get(self.status, 'white')
 
     def bold(self):
         # TESTING is bold for more contrast with REJECTED
-        return self.status in [Status.OK, Status.TESTING]
+        return self.status in [Status.OK, Status.TESTING, Status.CHECK_FAILED, Status.DISQUALIFIED]
 
     def table_score(self):
         if self.status in [Status.TESTING, Status.REJECTED] and self.score is None:
@@ -388,7 +436,8 @@ class Deadlines:
     @staticmethod
     def parse(text, server_tz):
         """Parse datetime string (obtained from a problem page) and convert it to UTC"""
-        dt = datetime.strptime(text, TIME_FORMAT)
+        # YYYY//MM/DD [hh[:mm[:ss]]]
+        dt = datetime(*map(int, re.findall(r'\d+', text)))
         return dt.replace(tzinfo=server_tz).astimezone(MSK_TZ)
 
 
@@ -481,6 +530,7 @@ class FullProblem(SummaryProblem):
     def suffix(self):
         if self._suffix is not None:
             return self._suffix
+        # if no langs are available, guess by problem name
         pref = self.name.split('/', 1)[0]
         if pref == 'asm':
             return '.S'
@@ -490,14 +540,14 @@ class FullProblem(SummaryProblem):
     def parse_sample(html):
         input_data, output_data = None, None
 
-        input_title = html.find('h4', text='Input')
+        input_title = html.find('h4', string='Input')
         if input_title is not None:
             input_data = input_title.find_next('pre').text
             # Add trailing newline, like in vim
             if not input_data.endswith('\n'):
                 input_data += '\n'
 
-        output_title = html.find('h4', text='Output')
+        output_title = html.find('h4', string='Output')
         if output_title is not None:
             output_data = output_title.find_next('pre').text
             if not output_data.endswith('\n'):
@@ -514,7 +564,7 @@ class FullProblem(SummaryProblem):
         problem_info = html.find('table', {'class': 'line-table-wb'})
         if problem_info is None:
             return None
-        next_block = problem_info.find_next('h3', text='Submit a solution')
+        next_block = problem_info.find_next('h3', string='Submit a solution')
         if next_block is None:
             next_block = problem_info.find_next('h2')
 
@@ -556,18 +606,11 @@ class FullProblem(SummaryProblem):
         return html
 
     @staticmethod
-    def _lang_suf(lang_id):
-        # NOTE compiler ids may change
-        lang_id = int(lang_id)
-        if lang_id in [2, 28, 51, 57, 61]:
-            return '.c'
-        if lang_id in [3, 29, 52, 58, 62]:
-            return '.cpp'
-        if lang_id in [25, 54]:
-            return '.tar'
-        if lang_id in [66, 67, 101, 102]:
-            return '.S'
-        return None
+    def _lang_suf(lang_id: str):
+        try:
+            return Lang(int(lang_id)).suf
+        except ValueError:
+            return None
 
     @staticmethod
     def guess_suffix(html):
@@ -607,7 +650,7 @@ class FullProblem(SummaryProblem):
         for tag in self._html.find_all(['a', 'img']):
             url = tag['href'] if tag.name == 'a' else tag['src']
             parts = urlsplit(url)
-            if parts.netloc != Links.HOST:
+            if parts.netloc != Links.DOMAIN:
                 continue
             query = parse_qs(parts.query)
             if 'file' in query:
@@ -671,6 +714,9 @@ def ejudge_standings(session):
     name = title[:title.find('[') - 1]
 
     table = soup.find('table', class_='standings')
+    if table is None:  # contest not started
+        return Standings([], [], name)
+
     rows = table.find_all('tr')
 
     tasks = [
@@ -726,6 +772,8 @@ def to_task_score(contest, cell):
     status = Status.REVIEW if 'cell_attr_pr' in cell['class'] \
         else Status.REJECTED if 'cell_attr_rj' in cell['class'] \
         else Status.TESTING if 'cell_attr_tr' in cell['class'] \
+        else Status.CHECK_FAILED if 'cell_attr_cf' in cell['class'] \
+        else Status.DISQUALIFIED if 'cell_attr_dq' in cell['class'] \
         else Status.PARTIAL if score == '0' \
         else Status.OK if score is not None \
         else Status.NOT_SUBMITTED
@@ -899,6 +947,7 @@ def get_problem_info(problem, cache, session):
     run_penalty = 0  # may be incorrect for kr
     current_penalty = 0
     deadlines = Deadlines(None, None)
+    alt_soft_deadline = None
     full_score_found = False
 
     if problem_info is None:
@@ -915,10 +964,18 @@ def get_problem_info(problem, cache, session):
                 current_penalty = int(value.text)
             elif key.text == 'Next soft deadline:':
                 deadlines.soft = Deadlines.parse(value.text, get_server_tz(cache, session))
+            elif key.text == 'Date penalty formula:':
+                deadline, dp_value = value.text.rsplit(' ', 1)
+                if dp_value.isdigit():
+                    # If fixed formula is used, "next soft deadline" is not rendered (ejudge 3.9.1)
+                    alt_soft_deadline = Deadlines.parse(deadline, get_server_tz(cache, session))
             elif key.text == 'Deadline:':
                 deadlines.hard = Deadlines.parse(value.text, get_server_tz(cache, session))
+        if deadlines.soft is None:
+            deadlines.soft = alt_soft_deadline
 
     if not full_score_found:
+        # problems past deadline, something else?
         try:
             problem_status = session.api().problem_status(problem.extract_id()).get('problem', {})
         except APIError as e:
